import java.util.*;

public class Block extends AttachableElement  {
   private static int id=0;  // Block identification number
   private final float mass;
   private Vector2D pos_t;     // current position at time t
   private Vector2D pos_tPlusDelta;  // next position in delta time in future
   private Vector2D speed_t;   // speed at time t
   private Vector2D speed_tPlusDelta;   // speed in delta time in future
   private Vector2D a_t;    // acceleration at time t
   private Vector2D a_tMinusDelta;  // acceleration delta time ago;
   private Vector<Spring> springs;  // Vector can grow, arrays cannot.

   private Block(){   // nobody can create a block without state
     super(id++);
     mass=0;
   }

/*si trabajamos en 1D, la coordenada x vale 0*/
   public Block(float mass, float position, float speed){
      super(id++);
      this.mass = mass;
      pos_t        = new Vector2D((double)position);
      speed_t      = new Vector2D((double)speed);
      a_t          =new Vector2D();
      a_tMinusDelta=new Vector2D();
      springs = new Vector<Spring>();
   }
/*nueva en 2D*/
   public Block(float mass, Vector2D position, Vector2D speed){
      super(id++);
      this.mass = mass;
      pos_t        =new Vector2D(position);
      speed_t      =new Vector2D(speed);
      a_t          =new Vector2D();
      a_tMinusDelta=new Vector2D();
      springs = new Vector<Spring>();
   }
   public void attachSpring (Spring spring) {
     springs.add(spring);
   }
   public void detachSpring (Spring spring) {
     springs.remove(spring);
   }
   public Vector2D getPosition() {
      return new Vector2D(pos_t);
   }
   private Vector2D getNetForce(float g, float k) {
     Vector2D force=new Vector2D();
     for (Spring s: springs)
        force = force.plus(s.getForce(this));
     force = force.minus(0,mass*g);
     force = force.minus(speed_t.times(k));
     return(force);
   }
   public void setInitialState(float g, float k) {
      a_tMinusDelta = getNetForce(g,k).times(1/mass);
   }
   public void computeNextState(double delta_t, float g,float k) {
     a_t = getNetForce(g,k).times(1/mass);

//     speed_tPlusDelta = speed_t + (3*a_t - a_tMinusDelta)*delta_t/2;
     speed_tPlusDelta = speed_t.plus(a_t.times(3).minus(a_tMinusDelta).times(delta_t/2));

//   pos_tPlusDelta = pos_t   +  speed_t   *   delta_t    +   (a_t*4 - a_tMinusDelta)*delta_t*delta_t/6;
     pos_tPlusDelta = pos_t.plus(speed_t.times(delta_t)).plus((a_t.times(4).minus(a_tMinusDelta)).times(delta_t*delta_t/6));
   }
   public void updateState(){
     pos_t = pos_tPlusDelta;
     speed_t = speed_tPlusDelta;
     a_tMinusDelta = a_t;
   }
   public String getDescription() {
      return "Block #" + getId()+": (x,y)";
   }
   public String getState() {
      return getPosition()+"";
   }
}
