public class Spring extends PhysicsElement  {
   private static int id=0;  // Spring identification
   protected float restLength;
   private final float stiffness;
   protected AttachableElement a_end, b_end;

   private Spring(){   // nobody can create a block without state
      super(id++);
      stiffness=0;
   }
   public Spring(float restLength, float stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      a_end = b_end = null;
   }
   public void attachAEnd (AttachableElement ae) {
      a_end = ae;
      ae.attachSpring(this);
   }
   public void detachAEnd(){
      if (a_end == null) return;
      a_end.detachSpring(this);
      a_end=null;
   }
   public void attachBEnd (AttachableElement ae) {
      b_end = ae;
      ae.attachSpring(this);
   }
   public void detachBEnd(){
      if (b_end == null) return;
      b_end.detachSpring(this);
      b_end = null;
   }
   private double getLength() {
      if (a_end== null || b_end==null) return restLength;
      else 
         return a_end.getPosition().getDistance(b_end.getPosition());
   }
   public Vector2D getForce(AttachableElement ae) {
      double force = 0;
      Vector2D direction;
      if ((a_end == null) || (b_end == null))
         return new Vector2D();
      if ((ae != a_end) && (ae != b_end))
         return new Vector2D();
      double length = getLength();
      double stretch = Math.abs(length) - restLength;
      force = stretch*stiffness;
      direction = a_end.getPosition().minus(b_end.getPosition()).getDirection();
      if (length < 0) force *=(-1);
      if (ae == b_end) return direction.times(force);
      return direction.times((-1)*force);
   }
   public String getDescription() {
      return "Spring #"+ getId()+": (x0,y0),(x1,y1)";
   }
   public String getState() {
      String s;
      if (a_end != null) 
         s = a_end.getPosition()+",";
      else if (b_end != null)
              s = (b_end.getPosition().minus(restLength))+",";
            else  
              s = 0.0+",";  // up to here left end is determined 
      if (b_end != null)
         s += b_end.getPosition();
      else if (a_end != null)
               s += (a_end.getPosition().plus(restLength));
            else
               s += restLength;
      return s;
   }
}
